@@ -35,6 +35,7 @@ from watson_machine_learning_client.wml_client_error import WMLClientError
 
 import copy
 import os
+import sys
 
 '''
 .. module:: WatsonMachineLearningAPIClient
@@ -61,6 +62,19 @@ class WatsonMachineLearningAPIClient:
         self.__wsd_supported_version_list = ['1.0', '1.1', '2.0']
         self.__predefined_instance_type_list = ['icp', 'openshift', 'wml_local', 'wsd_local']
         os.environ['WSD_PLATFORM'] = 'False'
+
+        print("DEPRECATED!! This Watson Machine Learning client version is deprecated"
+              " starting Sep 1st and will be discontinued at the end of the migration period. "
+              "Refer to the documentation at 'https://dataplatform.cloud.ibm.com/docs/content/wsj/analyze-data/wml-ai.html' "
+              "for the migration process to be able to access new features")
+
+        if (3 == sys.version_info.major) and (6 == sys.version_info.minor):
+            print(
+                "DEPRECATED!! Python 3.6 framework is deprecated and will be removed on Jan 20th, 2021. It will be "
+                "read-only mode starting Nov 20th, 2020. i.e you won't be able to create new assets using this client. "
+                "For cloud, switch to using new client https://pypi.org/project/ibm-watson-machine-learning with "
+                "python 3.7")
+
         if wml_credentials is None:
             raise NoWMLCredentialsProvided()
         if self.wml_credentials['url'][-1] == "/":
