@@ -132,6 +132,11 @@ class Deployments(WMLResource):
                             artifact_details["entity"]["type"]:
                         print("Note: Model of framework tensorflow and versions 1.5/1.11 has been deprecated. These versions will not be supported after 26th Nov 2019.")
 
+                    if 'mllib_2.3' in artifact_details["entity"]["type"]:
+                        print(
+                            "NOTE!! DEPRECATED!! Spark 2.3 framework for Watson Machine Learning client is deprecated and will be removed on December 1, 2020."
+                            "Use Spark 2.4 instead. For details, see https://dataplatform.cloud.ibm.com/docs/content/wsj/analyze-data/pm_service_supported_frameworks.html ")
+
                 metaProps['asset'] = {'href': details}
                 if self._client.CAMS:
                     if self._client.default_space_id is not None:
@@ -335,10 +340,14 @@ class Deployments(WMLResource):
                 )
                 return self._handle_response(200, "deployment", response)
             else:
-                return self._get_artifact_details(url, deployment_uid, limit, 'deployments')
+                details = self._get_artifact_details(url, deployment_uid, limit, 'deployments')
+                if "system" in details:
+                    print("Note: " + details['system']['warnings'][0]['message'])
         else:
-            return self._get_artifact_details(url, deployment_uid, limit, 'deployments')
-
+            details =self._get_artifact_details(url, deployment_uid, limit, 'deployments')
+            if "system" in details:
+                print("Note: " + details['system']['warnings'][0]['message'])
+        return  details
     @staticmethod
     @docstring_parameter({'str_type': STR_TYPE_NAME})
     def get_scoring_href(deployment_details):
@@ -910,13 +919,15 @@ class Deployments(WMLResource):
             headers.update({'x-global-transaction-id': transaction_id})
         # making change - connection keep alive
         scoring_url = self._href_definitions.get_async_deployment_job_href()
+        import requests
         if not self._ICP:
-            response_scoring = self.session.post(
+            response_scoring = requests.post(
                 scoring_url,
                 json=scoring_payload,
                 headers=headers)
         else:
-            response_scoring = self.session.post(
+            #response_scoring = self.session.post(
+            response_scoring=requests.post(
                 scoring_url,
                 json=scoring_payload,
                 headers=headers,
