@@ -1,19 +1,19 @@
 from typing import TYPE_CHECKING, List, Dict
-import sys
 
-from pandas import DataFrame
+from pandas import DataFrame, MultiIndex
 
-from watson_machine_learning_client import WatsonMachineLearningAPIClient
+from watson_machine_learning_client.utils.autoai.enums import (
+    PredictionType, RegressionAlgorithms, ClassificationAlgorithms)
+from watson_machine_learning_client.utils.autoai.errors import PipelineNotLoaded, FitNeeded, InvalidPredictionType
+from watson_machine_learning_client.utils.autoai.progress_bar import ProgressBar
 from watson_machine_learning_client.utils.autoai.utils import (
     fetch_pipelines, is_ipython, RunDetailsGenerator, check_dependencies_versions, try_import_autoai_libs,
     create_summary)
-from watson_machine_learning_client.utils.autoai.progress_bar import ProgressBar
-from watson_machine_learning_client.utils.autoai.errors import PipelineNotLoaded, FitNeeded
-from watson_machine_learning_client.utils.autoai.enums import (
-    PredictionType, RegressionAlgorithms, ClassificationAlgorithms, Directions, MetricsToDirections)
+from watson_machine_learning_client.utils.autoai.watson_studio import change_client_get_headers
 from .base_engine import BaseEngine
 
 if TYPE_CHECKING:
+    from watson_machine_learning_client.workspace import WorkSpace
     from watson_machine_learning_client.helpers.connections import DataConnection
     from sklearn.pipeline import Pipeline
 
@@ -28,12 +28,13 @@ class WMLEngine(BaseEngine):
 
     Parameters
     ----------
-    wml_client: WatsonMachineLearningAPIClient, required
-        Wrapper instance for WML Client.
+    workspace: WorkSpace, required
+        WorkSpace object with wml client and all needed parameters.
     """
 
-    def __init__(self, wml_client: 'WatsonMachineLearningAPIClient') -> None:
-        self._wml_client = wml_client
+    def __init__(self, workspace: 'WorkSpace') -> None:
+        self.workspace = workspace
+        self._wml_client = workspace.wml_client
         self._auto_pipelines_parameters = None
         self._wml_pipeline_metadata = None
         self._wml_training_metadata = None
@@ -89,7 +90,8 @@ class WMLEngine(BaseEngine):
                                 "daub_include_only_estimators":
                                     self._auto_pipelines_parameters.get('daub_include_only_estimators'),
                                 'scorer_for_ranking': self._auto_pipelines_parameters.get('scoring'),
-                                'compute_pipeline_notebooks_flag': self._auto_pipelines_parameters.get('notebooks', False),
+                                'compute_pipeline_notebooks_flag': self._auto_pipelines_parameters.get('notebooks',
+                                                                                                       False),
                                 'run_cognito_flag': True,
                                 'holdout_param': self._auto_pipelines_parameters.get('test_size')
                             }
@@ -143,9 +145,10 @@ class WMLEngine(BaseEngine):
 
         # note: only pass positive label when scoring is binary
         if (self._auto_pipelines_parameters.get('positive_label')
-                and self._auto_pipelines_parameters.get('scoring') == PredictionType.BINARY):
+                and self._auto_pipelines_parameters.get('prediction_type') == PredictionType.BINARY):
             self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
-                'pipelines'][0]['nodes'][0]['parameters']['optimization']['positive_class'] = self._auto_pipelines_parameters.get('positive_label')
+                'pipelines'][0]['nodes'][0]['parameters']['optimization'][
+                'positive_label'] = self._auto_pipelines_parameters.get('positive_label')
         # --- end note
 
         # note: only pass excel_sheet when it is different than 0
@@ -157,9 +160,10 @@ class WMLEngine(BaseEngine):
 
         if not self._wml_client.ICP:
             self._wml_pipeline_metadata[
-                self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT]['runtimes'][0]['version'] = self._auto_pipelines_parameters['autoai_pod_version']
+                self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT]['runtimes'][0]['version'] = \
+                self._auto_pipelines_parameters['autoai_pod_version']
 
-        if self._wml_client.ICP_30:
+        else:
             del self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT]['runtimes'][0][
                 'app_data']['wml_data']
             self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT]['runtimes'][0][
@@ -169,6 +173,68 @@ class WMLEngine(BaseEngine):
                 }
             }
 
+        # note: OBM part, only if user specified OBM as a preprocess method
+        if self._auto_pipelines_parameters['data_join_graph']:
+            # note: remove kb params for OBM only training
+            if self._auto_pipelines_parameters.get('data_join_only'):
+                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                    'pipelines'][0]['nodes'] = []
+                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                    'runtimes'] = []
+            # --- end note
+            else:
+                # note: update autoai KB POD metadata
+                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                    'pipelines'][0]['nodes'][0]['inputs'] = [{"id": "kb_input",
+                                                              "links": [{"node_id_ref": "obm",
+                                                                         "port_id_ref": "obm_out"
+                                                                         }]
+                                                              }]
+
+                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                    'pipelines'][0]['nodes'][0]['outputs'] = [{"id": "outputData"
+                                                               }]
+
+                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                    'pipelines'][0]['nodes'][0]['id'] = 'kb'
+
+                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                    'pipelines'][0]['nodes'][0]['runtime_ref'] = 'kb'
+
+                self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                    'runtimes'][0]['id'] = 'kb'
+                # --- end note
+
+            # note: insert OBM experiment metadata into the first place of nodes
+            self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                'pipelines'][0]['nodes'].insert(0, self._auto_pipelines_parameters['data_join_graph'].to_dict())
+            # --- end note
+
+            # note: insert OBM runtime metadata into the first place
+            self._wml_pipeline_metadata[self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT][
+                'runtimes'].insert(0, {"id": "obm",
+                                       "name": "auto_ai.obm",
+                                       "app_data": {
+                                           "wml_data": {
+                                               "runtime_spec_v4": {
+                                                   "compute": {
+                                                       "name": self._auto_pipelines_parameters[
+                                                           'data_join_graph'].t_shirt_size,
+                                                       "nodes": 1
+                                                   }
+                                               }
+                                           }
+                                       }
+                                       }
+                                   )
+            # --- end note
+
+            if not self._wml_client.ICP:
+                self._wml_pipeline_metadata[
+                    self._wml_client.pipelines.ConfigurationMetaNames.DOCUMENT]['runtimes'][0]['version'] = \
+                    self._auto_pipelines_parameters['obm_pod_version']
+        # --- end note
+
     def _initialize_wml_training_metadata(self,
                                           training_data_reference: List['DataConnection'],
                                           training_results_reference: 'DataConnection') -> None:
@@ -277,8 +343,22 @@ class WMLEngine(BaseEngine):
         """
         self._initialize_wml_training_metadata(training_data_reference=training_data_reference,
                                                training_results_reference=training_results_reference)
+
+        # note: support for spark hummingbird
+        if 'auto_ai.obm' in str(self._wml_pipeline_metadata):
+            change_client_get_headers(client=self._wml_client.training._client,
+                                      _type='new',
+                                      workspace=self.workspace)
+
         run_params = self._wml_client.training.run(meta_props=self._wml_training_metadata,
                                                    asynchronous=True)
+
+        # note: support for spark hummingbird
+        if 'auto_ai.obm' in str(self._wml_pipeline_metadata):
+            change_client_get_headers(client=self._wml_client.training._client,
+                                      _type='old',
+                                      workspace=self.workspace)
+
         self._current_run_id = run_params['metadata']['guid']
 
         wml_pipeline_details = self.get_params()
@@ -328,6 +408,9 @@ class WMLEngine(BaseEngine):
             # note: only console version
             else:
                 total = 200 * int(wml_pipeline_details.get('max_num_daub_ensembles', 1))
+                if self._auto_pipelines_parameters['data_join_graph']:
+                    total = total * 2  # make progress bar larger when we are using OBM + KB
+
                 progress_bar = ProgressBar(desc="Total", total=total, position=0, ncols=100)
                 end = False
                 while not end:
@@ -420,17 +503,128 @@ class WMLEngine(BaseEngine):
         pipeline_parameters = {
             "composition_steps": [],
             "pipeline_nodes": [],
+            "features_importance": None,
         }
 
+        # note: retrieve all additional pipeline evaluation information
         for pipeline in run_params['entity']['status'].get('metrics', []):
-            if (pipeline['context']['phase'] == 'global_output' and
-                    pipeline['context']['intermediate_model']['name'].split('P')[-1] == pipeline_name.split('_')[-1]):
+            if pipeline['context']['intermediate_model']['name'].split('P')[-1] == pipeline_name.split('_')[-1]:
+
                 pipeline_parameters['composition_steps'] = pipeline['context']['intermediate_model'][
                     'composition_steps']
                 pipeline_parameters['pipeline_nodes'] = pipeline['context']['intermediate_model']['pipeline_nodes']
+                pipeline_parameters['ml_metrics'] = self._get_metrics(pipeline)
+
+                if (self._auto_pipelines_parameters['prediction_type'] == 'binary'
+                        or (self._auto_pipelines_parameters['prediction_type'] == 'classification'
+                            and 'binary_classification' in str(run_params))):
+
+                    pipeline_parameters['features_importance'] = self._get_features_importance(pipeline)
+                    pipeline_parameters['confusion_matrix'] = self._get_confusion_matrix(pipeline)
+                    pipeline_parameters['roc_curve'] = self._get_roc_curve(pipeline)
+
+                elif (self._auto_pipelines_parameters['prediction_type'] == 'multiclass'
+                      or (self._auto_pipelines_parameters['prediction_type'] == 'classification'
+                          and 'multi_class_classification' in str(run_params))):
+
+                    pipeline_parameters['features_importance'] = self._get_features_importance(pipeline)
+                    pipeline_parameters['roc_curve'] = self._get_one_vs_all(pipeline, 'roc_curve')
+                    pipeline_parameters['confusion_matrix'] = self._get_one_vs_all(pipeline, 'confusion_matrix')
+
+                elif self._auto_pipelines_parameters['prediction_type'] == 'regression':
+                    pipeline_parameters['features_importance'] = self._get_features_importance(pipeline)
+
+                else:
+                    raise InvalidPredictionType(self._auto_pipelines_parameters['prediction_type'],
+                                                reason="Prediction type not recognized.")
+
+        # --- end note
 
         return pipeline_parameters
 
+    @staticmethod
+    def _get_metrics(pipeline_details: Dict) -> 'DataFrame':
+        """Retrieve evaluation metrics data from particular pipeline details."""
+        data = pipeline_details['ml_metrics']
+        columns = [field for field in data.keys()]
+        values = [[value for value in data.values()]]
+
+        metrics = DataFrame(data=values, columns=columns)
+        metrics.index = ['score']
+        metrics = metrics.transpose()
+
+        return metrics
+
+    @staticmethod
+    def _get_confusion_matrix(pipeline_details: Dict) -> 'DataFrame':
+        """Retrieve confusion matrix data from particular pipeline details. (Binary)"""
+        data = pipeline_details['context'].get('binary_classification',
+                                               {'confusion_matrix': [{}, {}]})['confusion_matrix']
+        columns = [name for name in data[0].keys()]
+        values = [[value for value in item.values()] for item in data]
+
+        confusion_matrix_data = DataFrame(data=values, columns=columns)
+        confusion_matrix_data.index = confusion_matrix_data['true_class']
+        confusion_matrix_data.drop(['true_class'], axis=1, inplace=True)
+
+        return confusion_matrix_data
+
+    @staticmethod
+    def _get_features_importance(pipeline_details: Dict) -> 'DataFrame':
+        """Retrieve features importance data from particular pipeline details."""
+        data = pipeline_details['context'].get('features_importance', [{'features': {}}])[0]['features']
+        columns = [name for name in data.keys()]
+        values = [[value for value in data.values()]]
+
+        features_importance_data = DataFrame(data=values, columns=columns, index=['features_importance'])
+        features_importance_data = features_importance_data.transpose()
+        features_importance_data.sort_values(by='features_importance', ascending=False, inplace=True)
+
+        return features_importance_data
+
+    @staticmethod
+    def _get_roc_curve(pipeline_details: Dict) -> 'DataFrame':
+        """Retrieve roc curve data from particular pipeline details. (Binary)"""
+        data = pipeline_details['context'].get('binary_classification', {'roc_curve': []})['roc_curve']
+        true_classes = [item['true_class'] for item in data]
+        values = [data[0]['fpr'], data[1]['fpr'], data[0]['thresholds'], data[1]['thresholds'], data[0]['tpr'],
+                  data[1]['tpr']]
+
+        index = MultiIndex.from_product([['fpr', 'thresholds', 'tpr'], true_classes],
+                                        names=['measurement types', 'true_class'])
+
+        roc_curve = DataFrame(data=values, index=index)
+
+        return roc_curve
+
+    @staticmethod
+    def _get_one_vs_all(pipeline_details: Dict, option: str) -> 'DataFrame':
+        """Retrieve roc curve or confusion matrix data from particular pipeline details. (Multiclass)"""
+        data = pipeline_details['context'].get('multi_class_classification', {'one_vs_all': []})['one_vs_all']
+
+        if option == 'confusion_matrix':
+            confusion_matrix_values = [item['confusion_matrix'].values() for item in data]
+            confusion_matrix_data = DataFrame(data=confusion_matrix_values,
+                                              columns=['fn', 'fp', 'tn', 'tp', 'true_class'])
+            confusion_matrix_data.index = confusion_matrix_data['true_class']
+            confusion_matrix_data.drop(['true_class'], axis=1, inplace=True)
+
+            return confusion_matrix_data
+
+        elif option == 'roc_curve':
+            true_classes = [item['roc_curve']['true_class'] for item in data]
+            values = [item['roc_curve']['fpr'] for item in data] + [item['roc_curve']['thresholds'] for item in
+                                                                    data] + [item['roc_curve']['tpr'] for item in data]
+            index = MultiIndex.from_product([['fpr', 'thresholds', 'tpr'], true_classes],
+                                            names=['measurement types', 'true_class'])
+            roc_curve = DataFrame(data=values, index=index)
+
+            return roc_curve
+
+        else:
+            raise ValueError(f"Wrong option for \"one_vs_all\": {option}. "
+                             f"Available options: [confusion_matrix, roc_curve]")
+
     def get_pipeline(self, pipeline_name: str, local_path: str = '.', persist: 'bool' = False) -> 'Pipeline':
         """
         Download specified pipeline from WML.
@@ -467,7 +661,7 @@ class WMLEngine(BaseEngine):
 
         except Exception as e:
             raise PipelineNotLoaded(
-                'Best Pipeline',
+                pipeline_name,
                 reason=f'Fit finished but there was some error during loading a pipeline from WML. Error: {e}')
 
         return pipelines.get(pipeline_name)
