@@ -431,7 +431,10 @@ class LocalAutoPipelines(BaseAutoPipelines):
         # note: try to download and load pipeline model from COS (auto-gen notebook scenario)
         if hasattr(self, '_result_client'):
             from watson_machine_learning_client.utils.autoai.utils import create_model_download_link, remove_file
-            from sklearn.externals import joblib
+            try:
+                from sklearn.externals import joblib
+            except ImportError:
+                import joblib
             filename = 'pipeline_model_auto-gen_notebook.pickle'
 
             try:
