@@ -1,19 +1,27 @@
 from copy import deepcopy
 from typing import TYPE_CHECKING, List, Union
+from warnings import warn
 
 from numpy import ndarray
 from pandas import DataFrame
 
-from watson_machine_learning_client.helpers.connections import DataConnection, S3Location, FSLocation, DSLocation
-from watson_machine_learning_client.utils.autoai.utils import try_import_lale
-from watson_machine_learning_client.utils.autoai.errors import FitNotCompleted
+from watson_machine_learning_client.helpers.connections import (
+    DataConnection, S3Location, FSLocation, CP4DAssetLocation, WMLSAssetLocation, WSDAssetLocation)
+from watson_machine_learning_client.preprocessing import DataJoinPipeline
 from watson_machine_learning_client.utils.autoai.enums import (
-    RunStateTypes, PipelineTypes, TShirtSize, ClassificationAlgorithms, RegressionAlgorithms)
+    RunStateTypes, PipelineTypes, TShirtSize, ClassificationAlgorithms, RegressionAlgorithms, DataConnectionTypes)
+from watson_machine_learning_client.utils.autoai.errors import (
+    FitNotCompleted, MissingDataPreprocessingStep, WrongDataJoinGraphNodeName, DataSourceSizeNotSupported, TrainingDataSourceIsNotFile)
+from watson_machine_learning_client.utils.autoai.utils import try_import_lale
+from watson_machine_learning_client.utils.autoai.wsd_ui import (
+    save_computed_pipelines_for_ui, save_experiment_for_ui, save_metadata_for_ui)
+from watson_machine_learning_client.utils.autoai.connection import validate_source_data_connections, validate_results_data_connection
 from .base_auto_pipelines import BaseAutoPipelines
 
 if TYPE_CHECKING:
     from watson_machine_learning_client.experiment.autoai.engines import WMLEngine
     from watson_machine_learning_client.utils.autoai.enums import Metrics, PredictionType
+    from watson_machine_learning_client.preprocessing import DataJoinGraph
     from sklearn.pipeline import Pipeline
 
 __all__ = [
@@ -76,6 +84,10 @@ class RemoteAutoPipelines(BaseAutoPipelines):
     engine: WMLEngine, required
         Engine for remote work on WML.
 
+    data_join_graph: DataJoinGraph, optional
+        A graph object with definition of join structure for multiple input data sources.
+        Data preprocess step for multiple files.
+
     """
 
     def __init__(self,
@@ -90,11 +102,14 @@ class RemoteAutoPipelines(BaseAutoPipelines):
                  t_shirt_size: 'TShirtSize' = TShirtSize.M,
                  train_sample_rows_test_size: float = None,
                  daub_include_only_estimators: List[Union['ClassificationAlgorithms', 'RegressionAlgorithms']] = None,
+                 data_join_graph: 'DataJoinGraph' = None,
                  csv_separator: Union[List[str], str] = ',',
                  excel_sheet: Union[str, int] = 0,
                  positive_label: str = None,
+                 data_join_only: bool = False,
                  notebooks=False,
-                 autoai_pod_version=None):
+                 autoai_pod_version=None,
+                 obm_pod_version=None):
         self.params = {
             'name': name,
             'desc': desc if desc else '',
@@ -106,11 +121,14 @@ class RemoteAutoPipelines(BaseAutoPipelines):
             't_shirt_size': t_shirt_size,
             'train_sample_rows_test_size': train_sample_rows_test_size,
             'daub_include_only_estimators': daub_include_only_estimators,
+            'data_join_graph': data_join_graph or False,
             'csv_separator': csv_separator,
             'excel_sheet': excel_sheet,
             'positive_label': positive_label,
+            'data_join_only': data_join_only,
             'notebooks': notebooks,
-            'autoai_pod_version': autoai_pod_version
+            'autoai_pod_version': autoai_pod_version,
+            'obm_pod_version': obm_pod_version,
         }
         self._engine: 'WMLEngine' = engine
         self._engine.initiate_remote_resources(params=self.params)
@@ -159,7 +177,9 @@ class RemoteAutoPipelines(BaseAutoPipelines):
         """
         _params = self._engine.get_params().copy()
         del _params['autoai_pod_version']
+        del _params['obm_pod_version']
         del _params['notebooks']
+        del _params['data_join_only']
 
         return _params
 
@@ -177,12 +197,6 @@ class RemoteAutoPipelines(BaseAutoPipelines):
 
         Parameters
         ----------
-        train_data: pandas.DataFrame, optional
-            You can pass a training data to AutoAI fit, it will be stored in your first training_data_reference
-            DataConnection COS storage under the bucket and path specified there.
-            If there is already some data stored under the specified 'path' and 'bucket',
-            this pandas DataFrame will override stored CSV in your COS.
-
         training_data_reference: List[DataConnection], required
             Data storage connection details to inform where training data is stored.
 
@@ -228,6 +242,20 @@ class RemoteAutoPipelines(BaseAutoPipelines):
         >>>     background_mode=False)
         """
 
+        # note: check if DataJoinGraph node names are correct and equivalent
+        # to training DataConnections IDs/data_join_node_names
+        if self.params['data_join_graph']:
+            data_join_graph = self.params['data_join_graph']
+            data_connection_ids = [connection.id for connection in training_data_reference]
+            for node_name in [node.table.name for node in data_join_graph.nodes]:
+                if node_name not in data_connection_ids:
+                    raise WrongDataJoinGraphNodeName(
+                        node_name,
+                        reason=f"Please make sure that each particular node name in data_join_graph is the same as "
+                               f"\"data_join_node_name\" parameter in particular equivalent training DataConnection. "
+                               f"The default names are taken as a DataConnection.location.path.")
+        # --- end note
+
         # note: update each training data connection with pipeline parameters for holdout split recreation
         for data_connection in training_data_reference:
             data_connection.auto_pipeline_params = self._engine._auto_pipelines_parameters
@@ -241,25 +269,18 @@ class RemoteAutoPipelines(BaseAutoPipelines):
         else:
             raise TypeError("train_data should be of type pandas.DataFrame")
 
-        for training_connection in training_data_reference:
-            # note: set project id when we are working on CP4D and using Data Assets Location
-            if isinstance(training_connection.location, DSLocation):
-                if self._workspace.WMLS:
-                    training_connection.location.href = training_connection.location.href.format(
-                        option='space_id',
-                        id=self._engine._wml_client.default_space_id)
-                else:
-                    training_connection.location.href = training_connection.location.href.format(
-                        option='project_id',
-                        id=self._engine._wml_client.default_project_id)
-            # -- end note
+        self._validate_training_data_size(training_data_reference)
+
+        training_data_reference = validate_source_data_connections(training_data_reference, workspace=self._workspace,
+                                                                   deployment=False)
+
         # note: if user did not provide results storage information, use default ones
         if training_results_reference is None:
             if isinstance(training_data_reference[0].location, S3Location):
                 training_results_reference = DataConnection(
                     connection=training_data_reference[0].connection,
                     location=S3Location(bucket=training_data_reference[0].location.bucket,
-                                        path='')
+                                        path='.')
                 )
 
             else:
@@ -275,11 +296,38 @@ class RemoteAutoPipelines(BaseAutoPipelines):
                     location=location
                 )
         # -- end note
+        # note: results can be stored only on FS or COS
+        if not isinstance(training_results_reference.location, (S3Location, FSLocation)):
+            raise TypeError('Unsupported results location type. Results referance can be stored'
+                            ' only on S3Location or FSLocation.')
+        # -- end
+
+        # note: only if we are going with OBM + KB scenario, add ID to results DataConnection
+        if self.params.get('data_join_graph'):
+            training_results_reference.id = 'outputData'
+        # --- end note
 
         run_params = self._engine.fit(training_data_reference=training_data_reference,
                                       training_results_reference=training_results_reference,
                                       background_mode=background_mode)
 
+        if isinstance(training_data_reference[0].location, WSDAssetLocation):
+            try:
+                wml_pipeline_details = self._workspace.wml_client.pipelines.get_details(
+                    run_params['entity']['pipeline']['id'])
+                save_experiment_for_ui(wml_pipeline_details,
+                                       run_params,
+                                       training_data_reference[0].location._local_asset_path,
+                                       training_data_reference[0].location._asset_id,
+                                       training_data_reference[0].location._asset_name)
+                save_computed_pipelines_for_ui(self._workspace.wml_client,
+                                               self._engine._current_run_id)
+                save_metadata_for_ui(wml_pipeline_details, run_params)
+
+            except Exception as e:
+                print(f"Cannot save experiment locally. It will not be visible in the WSD UI. Error: {e}")
+                warn(f"Cannot save experiment locally. It will not be visible in the WSD UI. Error: {e}")
+
         return run_params
 
     #####################
@@ -493,3 +541,81 @@ class RemoteAutoPipelines(BaseAutoPipelines):
             data_connection._run_id = self._engine._current_run_id
 
         return data_connections
+
+    def _validate_training_data_size(self, training_data_reference: List['DataConnection']) -> None:
+        """
+        Check size of dataset in training data connection
+        """
+
+        for training_connection in training_data_reference:
+            t_shirt_size = self._engine._auto_pipelines_parameters.get('t_shirt_size')
+
+            if isinstance(training_connection.location, S3Location):
+                size = training_connection.location._get_file_size(training_connection._init_cos_client())
+            elif isinstance(training_connection.location, WSDAssetLocation):
+                size = training_connection.location._get_file_size()
+            else:
+                size = training_connection.location._get_file_size(self._workspace)
+
+            if size is None:
+                raise TrainingDataSourceIsNotFile(training_connection.location)
+
+            if self._workspace.WMLS:
+                if t_shirt_size in (TShirtSize.S, TShirtSize.M) and size > 100 * 1024 * 1024:
+                    raise DataSourceSizeNotSupported()
+                elif t_shirt_size in (TShirtSize.ML, TShirtSize.L) and size > 1024 * 1024 * 1024:
+                    raise DataSourceSizeNotSupported()
+            else:
+                pass # note: data source size checking for other environment is not implemented
+              
+    def get_preprocessed_data_connection(self) -> 'DataConnection':
+        """
+        Create DataConnection object for further user usage (with OBM output)
+            (eg. to handle data storage connection or to recreate autoai holdout split).
+
+        Returns
+        -------
+        DataConnection with populated optimizer parameters
+        """
+        optimizer_parameters = self.get_params()
+
+        if optimizer_parameters['data_join_graph']:
+            details = self._engine._wml_client.training.get_details(
+                training_uid=self._engine._current_run_id)
+            if self.params['data_join_only']:
+                path = f"{details['entity']['results_reference']['location']['path']}/{self._engine._current_run_id}"
+            else:
+                path = details['entity']['status']['metrics'][0]['context']['intermediate_model'][
+                    'location']['model'].split('/data/kb/')[0]
+            path = f"{path}/data/obm/features/part"
+
+            results_connection = DataConnection._from_dict(_dict=details['entity']['results_reference'])
+            results_connection.auto_pipeline_params = deepcopy(optimizer_parameters)
+            results_connection._wml_client = self._engine._wml_client
+            results_connection._run_id = self._engine._current_run_id
+            results_connection._obm = True  # indicator for OBM output data
+            results_connection._obm_cos_path = path
+
+            return results_connection
+
+        else:
+            raise MissingDataPreprocessingStep(
+                reason="Cannot get preprocessed data as preprocessing step was not performed.")
+
+    def get_preprocessing_pipeline(self) -> 'DataJoinPipeline':
+        """
+        Returns preprocessing pipeline object for further usage.
+            (eg. to visualize preprocessing pipeline as graph).
+
+        Returns
+        -------
+        DataJoinPipeline
+        """
+        optimizer_parameters = self.get_params()
+
+        if optimizer_parameters['data_join_graph']:
+            return DataJoinPipeline(self.get_preprocessed_data_connection(), self)
+
+        else:
+            raise MissingDataPreprocessingStep(
+                reason="Cannot get preprocessed pipeline as preprocessing step was not performed.")
