@@ -185,14 +185,42 @@ score = {}()
         try:
 
             if self._client.WSD:
-                if self.ConfigurationMetaNames.RUNTIME_UID not in meta_props:
-                    raise WMLClientError('Missing RUNTIME_UID in input meta_props.')
+                if self._client.WSD_20:
+                    if self.ConfigurationMetaNames.RUNTIME_UID not in meta_props and self.ConfigurationMetaNames.SOFTWARE_SPEC_UID not in meta_props:
+                        raise WMLClientError("Invalid input. It is mandatory to provide RUNTIME_UID or "
+                                             "SOFTWARE_SPEC_UID in meta_props. RUNTIME_UID is deprecated")
+                else:
+                    if self.ConfigurationMetaNames.RUNTIME_UID not in meta_props:
+                        raise WMLClientError('Missing RUNTIME_UID in input meta_props.')
                 if self.ConfigurationMetaNames.TYPE not in meta_props:
                     meta_props.update({self.ConfigurationMetaNames.TYPE: 'python'})
+
                 import copy
                 function_metadata = self.ConfigurationMetaNames._generate_resource_metadata(meta_props,
                                                                                             with_validation=True,
-                                                                                            client=self._client)
+                                                                                           client=self._client)
+                input_schemas = []
+                output_schemas = []
+                if self.ConfigurationMetaNames.INPUT_DATA_SCHEMAS in meta_props and \
+                        meta_props[self.ConfigurationMetaNames.INPUT_DATA_SCHEMAS] is not None:
+                    input_schemas = meta_props[self.ConfigurationMetaNames.INPUT_DATA_SCHEMAS]
+                    function_metadata.pop(self.ConfigurationMetaNames.INPUT_DATA_SCHEMAS)
+                if self.ConfigurationMetaNames.OUTPUT_DATA_SCHEMAS in meta_props and \
+                        meta_props[self.ConfigurationMetaNames.OUTPUT_DATA_SCHEMAS] is not None:
+                    output_schemas = meta_props[self.ConfigurationMetaNames.OUTPUT_DATA_SCHEMAS]
+                    function_metadata.pop(self.ConfigurationMetaNames.OUTPUT_DATA_SCHEMAS)
+                if len(input_schemas) != 0 or len(output_schemas) != 0:
+                    function_metadata.update({"schemas": {
+                        "input": input_schemas,
+                        "output": output_schemas}
+                    })
+                if self._client.WSD_20:
+                    if self.ConfigurationMetaNames.SOFTWARE_SPEC_UID in meta_props and \
+                            meta_props[self.ConfigurationMetaNames.SOFTWARE_SPEC_UID] is not None:
+                        function_metadata.pop('software_spec')
+                        function_metadata.update(
+                            {"software_spec": {"base_id":meta_props[self.ConfigurationMetaNames.SOFTWARE_SPEC_UID]}})
+
 
                 if self._client.default_project_id is not None:
                     function_metadata['project'] = {'href': "/v2/projects/" + self._client.default_project_id}
@@ -822,7 +850,7 @@ score = {}()
 
         if self._client.ICP_30 is None:
             raise WMLClientError(
-                u'Revision support is not there in this version WML server. It is supported only from 3.1.0 onwards.')
+                u'Revision APIs are not supported in this release.')
         else:
             url = self._href_definitions.get_functions_href()
             return self._create_revision_artifact(url, function_uid, 'functions')
@@ -852,7 +880,7 @@ score = {}()
 
         if not self._client.ICP_30:
             raise WMLClientError(
-                'Not supported. Revisions APIs are supported only for WML Cloud Pack for Data 3.0 and above.')
+                'Revision APIs are not supported in this release. ')
         else:
             url = self._href_definitions.get_function_href(function_uid)
             return self._get_with_or_without_limit(url, limit=None, op_name="model",
@@ -882,7 +910,7 @@ score = {}()
 
         if self._client.ICP_30 is None:
             raise WMLClientError(
-                u'Revision support is not there in this WML server. It is supported only from 3.1.0 onwards.')
+                u'Revision support is not there in this release.')
         else:
             url = self._href_definitions.get_function_href(function_uid)
             function_resources = self._get_artifact_details(url, "revisions", limit, 'function revisions')[u'resources']
