@@ -153,29 +153,53 @@ class Script(WMLResource):
         #check if the software spec specified is base or derived and
         # accordingly update the entity for asset creation
 
+        # TODO WSD2.0
+        #kaganesa: remove the below if and else. once v2/software_spec apis are available in WSD2.0,
+        # everything inside if should work.
+        if not self._client.WSD:
 
-        sw_spec_details = self._client.software_specifications.get_details(script_meta[u'software_spec_uid'])
+            sw_spec_details = self._client.software_specifications.get_details(script_meta[u'software_spec_uid'])
 
-        if(sw_spec_details[u'entity'][u'software_specification'][u'type'] == 'base'):
-            asset_meta = {
-                "metadata": {
-                    "name": name,
-                    "description": desc,
-                    "asset_type": "script",
-                    "origin_country": "us",
-                    "asset_category": "USER"
-                },
-                "entity": {
-                    "script": {
-                        "language": {
-                            "name": "python3"
-                        },
-                        "software_spec": {
-                            "base_id": script_meta[u'software_spec_uid']
+            if(sw_spec_details[u'entity'][u'software_specification'][u'type'] == 'base'):
+                asset_meta = {
+                    "metadata": {
+                        "name": name,
+                        "description": desc,
+                        "asset_type": "script",
+                        "origin_country": "us",
+                        "asset_category": "USER"
+                    },
+                    "entity": {
+                        "script": {
+                            "language": {
+                                "name": "python3"
+                            },
+                            "software_spec": {
+                                "base_id": script_meta[u'software_spec_uid']
+                            }
+                        }
+                    }
+                }
+            else:
+                asset_meta = {
+                    "metadata": {
+                        "name": name,
+                        "description": desc,
+                        "asset_type": "script",
+                        "origin_country": "us",
+                        "asset_category": "USER"
+                    },
+                    "entity": {
+                        "script": {
+                            "language": {
+                                "name": "python3"
+                            },
+                            "software_spec": {
+                                "id": script_meta[u'software_spec_uid']
+                            }
                         }
                     }
                 }
-            }
         else:
             asset_meta = {
                 "metadata": {
@@ -208,7 +232,7 @@ class Script(WMLResource):
             meta_props = {
                     "name": name
             }
-            details = Script._wsd_create_asset(self, "script", input_payload, meta_props, file_path)
+            details = Script._wsd_create_asset(self, "script", asset_meta, meta_props, file_path, user_archive_file=True)
             return self._get_required_element_from_response(details)
         else:
             if not self._ICP:
@@ -726,6 +750,10 @@ class Script(WMLResource):
             response = requests.delete(self._href_definitions.get_asset_href(asset_uid), params=self._client._params(),
                                     headers=self._client._get_headers())
         else:
+            if not self._client.WSD and self._if_deployment_exist_for_asset(asset_uid):
+                raise WMLClientError(
+                    u'Cannot delete scripts that has existing deployments. Please delete all associated deployments and try again')
+
             response = requests.delete(self._href_definitions.get_asset_href(asset_uid), params=self._client._params(),
                                       headers=self._client._get_headers(), verify=False)
         if response.status_code == 200:
@@ -854,7 +882,6 @@ class Script(WMLResource):
             response = requests.delete(self._href_definitions.get_asset_href(asset_uid), params=self._client._params(),
                                        headers=self._client._get_headers(), verify=False)
 
-
     def _get_required_element_from_response(self, response_data):
 
         WMLResource._validate_type(response_data, u'scripts', dict)
@@ -900,22 +927,9 @@ class Script(WMLResource):
                                 'name': response_data['metadata']['name'],
                                 'href': href,
                                 'asset_type': response_data['metadata']['asset_type'],
-                                'created_at': response_data['metadata']['created_at'],
-                                'last_updated_at': response_data['metadata']['usage']['last_updated_at']
+                                'created_at': response_data['metadata']['created_at']
                                 }
 
-                    if self._client.ICP_30 is not None:
-                        if "revision_id" in response_data[u'metadata']:
-                            revision_id = response_data[u'metadata'][u'revision_id']
-                            metadata.update({'revision_id': response_data[u'metadata'][u'revision_id']})
-
-                        if "attachments" in response_data and response_data[u'attachments']:
-                            metadata.update({'attachment_id': response_data[u'attachments'][0][u'id']})
-
-                        if "commit_info" in response_data[u'metadata'] and revision_id is not None:
-                            metadata.update(
-                                {'revision_commit_date': response_data[u'metadata'][u'commit_info']['committed_at']})
-
                     if 'description' in response_data[u'metadata']:
                         metadata.update(
                             {'description': response_data[u'metadata'][u'description']})
@@ -923,6 +937,23 @@ class Script(WMLResource):
                     new_el = {'metadata': metadata,
                               'entity': response_data['entity']
                               }
+                    if self._client.WSD_20 is not None:
+                        if "revision_id" in response_data[u'metadata']:
+                            revision_id = response_data[u'metadata'][u'revision_id']
+                            new_el['metadata'].update({'revision_id': response_data[u'metadata'][u'revision_id']})
+
+                        if "attachments" in response_data and response_data[u'attachments']:
+                            new_el['metadata'].update({'attachment_id': response_data[u'attachments'][0][u'id']})
+
+                        if "commit_info" in response_data[u'metadata'] and revision_id is not None:
+                            new_el['metadata'].update(
+                                {'revision_commit_date': response_data[u'metadata'][u'commit_info']['committed_at']})
+
+                    if 'usage' in response_data['metadata']:
+                        new_el['metadata'].update({'last_updated_at': response_data['metadata']['usage']['last_updated_at']})
+                    else:
+                        new_el['metadata'].update(
+                         {'last_updated_at': response_data['metadata']['last_updated_at']})
                 else:
                     metadata = {'project_id': response_data['metadata']['project_id'],
                                 'guid': response_data['metadata']['asset_id'],
@@ -949,8 +980,7 @@ class Script(WMLResource):
                             {'description': response_data[u'metadata'][u'description']})
                     new_el = {'metadata': metadata,
                               'entity': response_data['entity']
-
-                            }
+                              }
             return new_el
         except Exception as e:
             raise WMLClientError("Failed to read Response from down-stream service: " + response_data.text)
