@@ -115,7 +115,10 @@ class TestAutoPipelinesRuns(unittest.TestCase):
         self.assertIsInstance(self.pipeline, Pipeline, msg="Fetched pipeline is not of sklearn type.")
 
     def test_15__get_pipeline_and_persist_it(self):
-        from sklearn.externals import joblib
+        try:
+            from sklearn.externals import joblib
+        except ImportError:
+            import joblib
         pipeline = self.optimizer.get_pipeline(pipeline_name="Pipeline_1",
                                                astype=AutoAI.PipelineTypes.SKLEARN,
                                                persist=True)
