@@ -1,5 +1,8 @@
 import unittest
-from sklearn.externals import joblib
+try:
+    from sklearn.externals import joblib
+except ImportError:
+    import joblib
 
 from watson_machine_learning_client.helpers import pipeline_to_script
 
